 /**
  * @author Roman Lim
  */
#ifndef NEIGHBOURSYNC_H
#define NEIGHBOURSYNC_H

#include <AM.h>

typedef struct {
  am_addr_t address;
  uint32_t lastSync;
  uint16_t usageCount;
  uint8_t failCount;
  uint16_t lplPeriod; // in ms
  uint16_t drift;
  bool positiveDrift;
} neighbour_sync_item_t;


typedef nx_struct {
  nx_uint16_t wakeupOffset;
  nx_uint16_t lplPeriod;	// this field contains REQ_SYNC_FLAG (highest bit) and lpl information
} neighbour_sync_header_t;


enum {
  NEIGHBOURSYNCTABLESIZE = 10, // size of sync neighbour table
  NO_VALID_OFFSET = 0xffff,
  NO_VALID_DRIFT = 0xffff,
  NO_ENTRY = 0xff,
  NO_SYNC = 0xffffffff,
  T32KHZ_TO_TMILLI_SHIFT = 5, // factor for conversion TMilli > T32khz
  RADIO_STARTUP_OFFSET = 128, // jiffys until the radio is started
  SEND_OFFSET = 0,  // less means: transmission begins later
  REVERSE_SEND_OFFSET = 50,  // less means: transmission begins earlier 
                             // 40, if no drift
  NO_COMPENSATION_OFFSET = 50,  // less means: transmission begins later. This offset is used to begin transmission earlier wehn no compensation is done
  ALARM_OFFSET = RADIO_STARTUP_OFFSET + SEND_OFFSET,
  AGING_PERIOD = 10,
  TOTAL_AGING_PERIOD = NEIGHBOURSYNCTABLESIZE * AGING_PERIOD, // after AGING_PERIOD packets, each usageCounter is decremented by AGING_PERIOD / 16
  LPL_DUTYCYCLE_CRITICAL_PERIOD = 130, // jiffys after radio startup in which lplc does not start radio

  SYNC_FAIL_THRESHOLD = 10, // after SYNC_FAIL_THRESHOLD non-acked packets, sync information is not valid anymore
  REQ_SYNC_FLAG = 0x8000,
  SYNC_TIMER_PERIOD = 30000UL, // period to gather sync-requests in milliseconds
};

#endif
